@@ -88,12 +88,13 @@ class BigQueryExtractor(BaseExtractor):
 
             logger.info(f"Getting table DDL for {dataset_ref}")
             table_ddl = client.query(
-                f"select table_name, ddl from `{dataset_ref.project}.{dataset_ref.dataset_id}.INFORMATION_SCHEMA.TABLES`"
+                f"select table_name, ddl from `{dataset_ref.project}.{dataset_ref.dataset_id}.INFORMATION_SCHEMA.TABLES`",
+                project=config.job_project_id,
             ).result()
 
             for table_name, ddl in table_ddl:
-                table = tables[str(table_name).lower()]
-                if not table:
+                table = tables.get(str(table_name).lower())
+                if table is None:
                     logger.error(f"table {table_name} not found for DDL")
                     continue
                 table.schema.sql_schema.table_schema = ddl
